from dataclasses import dataclass
from typing import Optional


@dataclass
class Style:
    """Style class.

    This class represents a style definition for a Mermaid diagram.

    Attributes:
        name (str): The name of the style definition.
        fill (str): The fill color of the style definition.
        color (str): The text color of the style definition.
        font_weight (str): The font weight of the style definition.
        stroke_width (str): The stroke width of the style definition.
        stroke (str): The stroke color of the style definition.
        other (str): Any other style definition sperated by ,.
    """
    name: str
    fill: Optional[str] = None
    color: Optional[str] = None
    font_weight: Optional[str] = None
    stroke_width: Optional[str] = None
    stroke: Optional[str] = None
    other: Optional[str] = None

    def __str__(self) -> str:
        """Return the string representation of the style definition.
        """
        string: str = f'classDef {self.name} '
        if self.fill:
            string += f',fill:{self.fill}'
        if self.color:
            string += f',color:{self.color}'
        if self.font_weight:
            string += f',font-weight:{self.font_weight}'
        if self.stroke_width:
            string += f',stroke-width:{self.stroke_width}'
        if self.stroke:
            string += f',stroke:{self.stroke}'
        if self.other:
            string += f',{self.other}'

        return string

    def __hash__(self) -> int:
        return hash(self.name)



"""Mindmap module.

This module provides the Mindmap class for creating and manipulating mindmap
diagrams.

Classes:
    Mindmap: Represents a mindmap diagram.
"""
from typing import Optional

from mermaid.configuration import Config
from mermaid.graph import Graph

from .level import Level, LevelShape


class Mindmap(Graph):
    """Mindmap class.

    This class represents a mindmap diagram.

    Args:
        title (str): The title of the mindmap.
        levels (Optional[list[Level]]): The levels of the mindmap. Defaults to None.
        shape (Optional[LevelShape]): The shape of the level. Defaults to None.
        config (Optional[Config]): The configuration for the mindmap. Defaults to None.
    """
    def __init__(self,
                 title: str,
                 levels: Optional[list[Level]] = None,
                 shape: Optional[LevelShape] = None,
                 config: Optional[Config] = None) -> None:
        """Initialize a new Mindmap.

        Args:
            title (str): The title of the mindmap.
            levels (Optional[list[Level]]): The levels of the mindmap. Defaults to None.
            shape (Optional[LevelShape]): The shape of the level. Defaults to None.
            config (Optional[Config]): The configuration for the mindmap. Defaults to None.
        """
        super().__init__(title, '', config)
        self.levels: list[Level] = levels if levels else []
        self.shape: LevelShape = shape if shape else LevelShape.DEFAULT
        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        script: str = '\nmindmap'
        script += f'\n\t{self.shape.start}{self.title}{self.shape.end}'
        script += '\n'
        for level in self.levels:
            script += f'{level}'

        # script += '\n'
        self.script += script


__all__ = ['Mindmap', 'Level', 'LevelShape']



import base64
from enum import Enum
from pathlib import Path
from typing import Union

import requests
from requests import Response

from .graph import Graph


class Position(Enum):
    """
    This class represents the position of the node in a Mermaid diagram.
    """
    LEFT = 'left'
    RIGHT = 'right'
    CENTER = 'center'
    NONE = 'none'


class Mermaid:
    """
    This class represents a Mermaid diagram.

    Attributes:
        _diagram (str): The base64 encoded string of the Mermaid diagram script.
        svg_response (Response): The response from the GET request to the Mermaid SVG API.
        img_response (Response): The response from the GET request to the Mermaid IMG API.
    """
    def __init__(self,
                 graph: Graph,
                 position: Union[Position, str] = Position.NONE):
        """
        The constructor for the Mermaid class.

        Parameters:
            graph (Graph): The Graph object containing the Mermaid diagram script.
        """
        self.__position: str = position if isinstance(position,
                                                      str) else position.value
        self._diagram = self._process_diagram(graph.script)
        self._make_request_to_mermaid()

    def set_position(self, position: Union[Position, str]) -> None:
        """
        Set the position of the node in the Mermaid diagram.

        Parameters:
            position (Union[Position, str]): The position of the node.
        """
        self.__position = position if isinstance(position,
                                                 str) else position.value

    @staticmethod
    def _process_diagram(diagram: str) -> str:
        """
        Process the Mermaid diagram script into a base64 encoded string.

        Parameters:
            diagram (str): The Mermaid diagram script.

        Returns:
            str: The base64 encoded string of the Mermaid diagram script.
        """
        graphbytes = diagram.encode('utf8')
        base64_bytes = base64.b64encode(graphbytes)
        diagram = base64_bytes.decode('ascii')
        return diagram

    def _repr_html_(self) -> str:
        """
        Return the text of the SVG response.

        Returns:
            str: The text of the SVG response.
        """
        if self.__position == Position.NONE.value:
            return self.svg_response.text
        return f'<div style="text-align:{self.__position}">{self.svg_response.text}</div>'

    def _make_request_to_mermaid(self) -> None:
        """
        Make GET requests to the Mermaid SVG and IMG APIs using
        the base64 encoded string of the Mermaid diagram script.
        """
        self.svg_response: Response = requests.get('https://mermaid.ink/svg/' +
                                                   self._diagram)
        self.img_response: Response = requests.get('https://mermaid.ink/img/' +
                                                   self._diagram)

    def to_svg(self, path: Union[str, Path]) -> None:
        """
        Write the SVG response text to a file.

        Parameters:
            path (Union[str, Path]): The path of the file to write to.
        """
        with open(path, 'w', encoding='utf-8') as file:
            file.write(self.svg_response.text)

    def to_png(self, path: Union[str, Path]) -> None:
        """
        Write the IMG response content to a file.

        Parameters:
            path (Union[str, Path]): The path of the file to write to.
        """
        with open(path, 'wb') as file:
            file.write(self.img_response.content)


try:
    from IPython import get_ipython
    if get_ipython() is not None:
        from IPython.core.magic import register_cell_magic
        from IPython.display import Image, display

        @register_cell_magic
        def mermaidjs(line, cell):
            options = line.strip().split()
            script: str = cell.strip()
            graph: Graph = Graph('mermaid diagram', script)
            mermaid: Mermaid = Mermaid(graph)
            if '--img' in options:
                display(Image(mermaid.img_response.content))
            else:
                display(Mermaid(graph))

        get_ipython().register_magic_function(mermaidjs, magic_kind='cell')
except ImportError:
    # TODO: add a suitible handler for exception.
    print('Error acured while importing mermaidjs .')



from enum import Enum
from typing import Union

from mermaid.sequence.element import Actor, Participant

ARROW_TYPES: dict[str, str] = {
    'Solid-line': '->',
    'Dotted-line': '-->',
    'Solid-arrow': '->>',
    'Dotted-arrow': '-->>',
    'Solid-cross': '-x',
    'Dotted-cross': '--x',
    'Solid-async': '-)',
    'Dotted-async': '--)'
}


class ArrowTypes(Enum):
    """Arrow types for mermaid sequence diagram."""
    SOLID_LINE = '->'
    DOTTED_LINE = '-->'
    SOLID_ARROW = '->>'
    DOTTED_ARROW = '-->>'
    SOLID_CROSS = '-x'
    DOTTED_CROSS = '--x'
    SOLID_ASYNC = '-)'
    DOTTED_ASYNC = '--)'


class Link:
    """Link class for mermaid sequence diagram.

    Args:
        source (ActorParticipant): Source of the link.
        target (ActorParticipant): Target of the link.
        type_ (Union[str,ArrowTypes]): Type of the link.
        message (str): Message to be displayed over the link.
        activate_target (bool, optional): Activate the target. Defaults to False.
        deactivate_target (bool, optional): Deactivate the target. Defaults to False.

    """
    def __init__(
        self,
        source: Union[Actor, Participant],
        target: Union[Actor, Participant],
        type_: Union[str, ArrowTypes],
        message: str,
        activate_target: bool = False,
        deactivate_target: bool = False,
    ) -> None:
        self.source: Union[Actor, Participant] = source
        self.target: Union[Actor, Participant] = target
        self.type_: Union[str, ArrowTypes] = ARROW_TYPES[type_] if isinstance(
            type_, str) else type_.value
        self.activate_target: bool = activate_target
        self.deactivate_target: bool = deactivate_target
        self.message: str = message

    def __str__(self) -> str:
        string: str = f'\t{self.source.id_}{self.type_}{self.target.id_}: {self.message}\n'
        if self.activate_target:
            string += f'activate {self.target.id_}\n'
        if self.deactivate_target:
            string += f'deactivate {self.target.id_}\n'

        return string



"""Mermaid module.

This module provides the Mermaid class for creating and manipulating Mermaid diagrams.
It also provides utility functions for loading data and converting text to snake case.

Classes:
    Mermaid: Represents a Mermaid diagram.

Functions:
    load(file_path): Load data from a file.
    text_to_snake_case(text): Convert a string of text to snake case.
"""
from enum import Enum

from .__main__ import Mermaid, Position
from .configuration import Config
from .graph import Graph
from .icon import Icon
from .style import Style
from .utils import load, text_to_snake_case

__version__: str = '0.5.2'


class Direction(Enum):
    """Enum for representing the direction of a Mermaid diagram."""
    LEFT_TO_RIGHT = 'LR'
    RIGHT_TO_LEFT = 'RL'
    TOP_TO_BOTTOM = 'TB'
    BOTTOM_TO_TOP = 'BT'


__all__ = [
    'Mermaid', 'load', 'Direction', 'Graph', 'Style', 'Config', 'Icon',
    'Position'
]



from typing import Union

from mermaid.sequence.link import Link


class Logic:
    """Logic class for mermaid sequence diagram."""
    pass


class Loop(Logic):
    """Loop class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        link (List[Link]): List of Link objects.
    """
    def __init__(self, condition: str, link: list[Link]):
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            link (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.link = link

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        return f'\tloop {self.condition}\n' + ''.join(
            [str(link) for link in self.link]) + '\tend\n'


class Alt(Logic):
    """Alt class for mermaid sequence diagram.

    Args:
        condition_links (dict[str, List[Link]]): Dictionary of condition
        and list of Link objects.
    """
    def __init__(self, condition_links: dict[str, list[Link]]):
        """Initialize alt.

        Args:
            condition_links (dict[str, List[Link]]): Dictionary of condition
            and list of Link objects.
        """
        self.condition_links = condition_links

    def __str__(self) -> str:
        """Return alt string.

        Returns:
            str: Alt string.
        """
        alt_str = ''
        n = 1
        for condition, links in self.condition_links.items():
            if n == 1:
                alt_str += f'\talt {condition}\n' + ''.join(
                    [str(link) for link in links])
                n += 1
            else:
                alt_str += f'\telse {condition}\n' + ''.join(
                    [str(link) for link in links])

        alt_str += '\tend\n'
        return alt_str


class Optional(Logic):
    """Optional class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        statements (List[Link]): List of Link objects.
    """
    def __init__(self, condition: str, statements: list[Link]) -> None:
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            statements (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.statements = statements

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        return f'\topt {self.condition}\n' + ''.join(
            [str(statement) for statement in self.statements]) + '\tend\n'


class Parallel:
    """Parallel class for mermaid sequence diagram.

    Args:
        condition_elements (dict[str, list[Union[Link, Logic]]]): Dictionary of
        condition and Link or Logic objects.
    """
    def __init__(
            self, condition_elements: dict[str, list[Union[Link,
                                                           Logic]]]) -> None:
        """Initialize parallel.

        Args:
            condition_elements (dict[str, list[Union[Link, Logic]]]): Dictionary of
            condition and Link or Logic objects.
        """
        self.condition_elements = condition_elements

    def __str__(self) -> str:
        """Return parallel string.

        Returns:
            str: Parallel string.
        """
        parallel_str = ''
        n = 1
        for condition, elements in self.condition_elements.items():
            if n == 1:
                parallel_str += f'\tpar {condition}\n' + ''.join(
                    [str(element) for element in elements])
                n += 1
            else:
                parallel_str += f'\tand {condition}\n' + ''.join(
                    [str(element) for element in elements])
        parallel_str += '\tend\n'
        return parallel_str


class Critical(Logic):
    """Critical class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        link (List[Link]): List of Link objects.
    """
    def __init__(
            self, condition: str, statements: list[Union[Link, Logic]],
            optional_statements: dict[str, list[Union[Link, Logic]]]) -> None:
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            link (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.statements = statements
        self.optional_statements = optional_statements

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        critical_str = f'\tcritical {self.condition}\n' + ''.join(
            [str(statement) for statement in self.statements])
        for condition, statements in self.optional_statements.items():
            critical_str += f'\toption {condition}\n' + ''.join(
                [str(statement) for statement in statements])
        critical_str += '\tend\n'
        return critical_str


class Break(Logic):
    """Break class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        link (List[Link]): List of Link objects.
    """
    def __init__(self, condition: str, statements: list[Union[Link,
                                                              Logic]]) -> None:
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            link (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.statements = statements

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        break_str = f'\tbreak {self.condition}\n' + ''.join(
            [str(statement) for statement in self.statements])
        break_str += '\tend\n'
        return break_str



from dataclasses import dataclass
from typing import Optional, Union

from mermaid import Direction, text_to_snake_case
from mermaid.style import Style


@dataclass
class NodeShape:
    """NodeShape class.

    This class represents the shape of a node in a flowchart.

    Attributes:
        start (str): The start character of the node shape.
        end (str): The end character of the node shape.
    """
    start: str
    end: str


NODE_SHAPES: dict[str, NodeShape] = {
    'normal': NodeShape('[', ']'),
    'round-edge': NodeShape('(', ')'),
    'stadium-shape': NodeShape('([', '])'),
    'subroutine-shape': NodeShape('[[', ']]'),
    'cylindrical': NodeShape('[(', ')]'),
    'circle': NodeShape('((', '))'),
    'label-shape': NodeShape('>', ']'),
    'rhombus': NodeShape('{', '}'),
    'hexagon': NodeShape('{{', '}}'),
    'parallelogram': NodeShape('[/', '/]'),
    'parallelogram-alt': NodeShape('[\\', '\\]'),
    'trapezoid': NodeShape('[/', '\\]'),
    'trapezoid-alt': NodeShape('[\\', '/]'),
    'double-circle': NodeShape('(((', ')))'),
}

HREF_TYPES: dict[str, str] = {
    'blank': '_blank',
    'self': '_self',
    'parent': '_parent',
    'top': '_top'
}


class Node:
    """Node class.

    This class represents a node in a flowchart.

    Attributes:
        id_ (str): The ID of the node.
        content (str): The content of the node.
        shape (str): The shape of the node.
        sub_nodes (list[Node]): The sub-nodes of the node.
        href (str): The hyperlink reference of the node.
        href_type (str): The type of the hyperlink reference of the node.
        styles (list[Style]): The styles of the node.
    """
    def __init__(self,
                 id_: str,
                 content: str = '',
                 shape: str = 'normal',
                 sub_nodes: Optional[list['Node']] = None,
                 href: Optional[str] = None,
                 href_type: str = 'blank',
                 styles: Optional[list[Style]] = None,
                 direction: Union[str, Direction] = 'LR') -> None:
        """Initialize a new Node.

        Args:
            id_ (str): The ID of the node.
            content (str): The content of the node.
            shape (str): The shape of the node.
            sub_nodes (Optional[list[Node]]): The sub-nodes of the node.
            href (Optional[str]): The hyperlink reference of the node.
            href_type (str): The type of the hyperlink reference of the node.
            styles (Optional[list[Style]]): The styles of the node.
            direction (Optional[Union[str,Direction]]): The direction of the node in case of a subgraph.
        """
        self.id_: str = text_to_snake_case(id_)
        self.content: str = content if content else id_
        self.shape: NodeShape = NODE_SHAPES[shape]
        self.href: str = href if href is not None else '#'
        self.href_type: str = HREF_TYPES[href_type]
        self.sub_nodes: list[
            'Node'] = sub_nodes if sub_nodes is not None else []
        self.styles: list[Style] = styles if styles is not None else []
        self.direction: str = direction if isinstance(direction,
                                                      str) else direction.value

    def __str__(self) -> str:
        """Return a string representation of the node.

        If the node has sub-nodes, it returns a string representation of a subgraph.
        Otherwise, it returns a string representation of a single node.

        Returns:
            str: A string representation of the node.
        """
        string: str = ''
        if len(self.sub_nodes):
            string = '\n'.join([
                f'subgraph {self.id_} ["{self.content}"]',
                f'\tdirection {self.direction}',
                '\n'.join([str(f'\t{node}') for node in self.sub_nodes]), 'end'
            ])
        else:
            string = ''.join([
                self.id_, self.shape.start, f'"{self.content}"', self.shape.end
            ])
            if self.href != '#':
                string = ''.join([
                    string, '\n',
                    f'click {self.id_} "{self.href}" {self.href_type}'
                ])

        for style in self.styles:
            string += f'\n{self.id_}:::{style.name}'
        return string



from enum import Enum
from typing import Optional


class Themes(Enum):
    """Enum class for themes

    args:
        Enum: Enum class

    methods:
        value: returns the value of the Enum class
    """
    DEFAULT = 'default'
    FOREST = 'forest'
    DARK = 'dark'
    NEUTRAL = 'neutral'
    BASE = 'base'


class Config:
    """Config class for mermaid diagrams

    args:
        theme: Themes = Themes.DEFAULT
        primary_color: Optional[str] = None
        primary_text_color: Optional[str] = None
        primary_border_color: Optional[str] = None
        line_color: Optional[str] = None
        secondary_color: Optional[str] = None
        tertiary_color: Optional[str] = None

    methods:
        __init__: initializes the class
        __str__: returns the string representation of the class
    """
    def __init__(self,
                 theme: Themes = Themes.DEFAULT,
                 primary_color: Optional[str] = None,
                 primary_text_color: Optional[str] = None,
                 primary_border_color: Optional[str] = None,
                 line_color: Optional[str] = None,
                 secondary_color: Optional[str] = None,
                 tertiary_color: Optional[str] = None) -> None:
        """Initializes the class

        args:
            theme: Themes = Themes.DEFAULT
            primary_color: Optional[str] = None
            primary_text_color: Optional[str] = None
            primary_border_color: Optional[str] = None
            line_color: Optional[str] = None
            secondary_color: Optional[str] = None
            tertiary_color: Optional[str] = None
        """
        self.theme: Themes = theme
        self.primary_color: Optional[str] = primary_color
        self.primary_text_color: Optional[str] = primary_text_color
        self.primary_border_color: Optional[str] = primary_border_color
        self.line_color: Optional[str] = line_color
        self.secondary_color: Optional[str] = secondary_color
        self.tertiary_color: Optional[str] = tertiary_color

    def __str__(self) -> str:
        """Returns the string representation of the class

        returns:
            str: string representation of the class
        """
        string: str = '%%{\n\tinit: {\n'

        string += f'\t\t"theme": "{self.theme.value}",\n\t\t"themeVariables": ' + '{\n'
        if self.primary_color:
            string += f'\t\t\t"primaryColor": "{self.primary_color}",\n'
        if self.primary_text_color:
            string += f'\t\t\t"primaryTextColor": "{self.primary_text_color}",\n'
        if self.primary_border_color:
            string += f'\t\t\t"primaryBorderColor": "{self.primary_border_color}",\n'
        if self.line_color:
            string += f'\t\t\t"lineColor": "{self.line_color}",\n'
        if self.secondary_color:
            string += f'\t\t\t"secondaryColor": "{self.secondary_color}",\n'
        if self.tertiary_color:
            string += f'\t\t\t"tertiaryColor": "{self.tertiary_color}",\n'
        if string.endswith(',\n'):
            string = string[:-2] + '\n'
        string += '\t\t}\n'
        string += '\t}\n}%%\n'
        return string



from dataclasses import dataclass


@dataclass
class Icon:
    name: str
    type_: str
    version: str = 'v1'

    def __str__(self) -> str:
        if self.version == 'v1':
            return f'{self.type_} {self.name}'
        elif self.version == 'v2':
            return f'{self.type_}:{self.name}'
        return f' {self.type_}:{self.name} '
